

///////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <EEPROM.h>
#include "AS5040.h" // библиотека связи с AS5040 по SSI

// на 2 ноге прерывание по датчику Холла заведенному на вертушку

// на 7 ноге CSN AS5040
#define byCSN 7
// на 8 ноге CLK AS5040
#define byCLK 8
// на 9 ноге D0 AS5040
#define byD0 9
// на 10 ноге PRG AS5040
#define byPGR 10

char stAsk[5] = ""; // строка для обмена с интерфейсом


AS5040 myAS5040(9, 8, 7); // data, clock, chip select, prg

unsigned int WindZeroAngle = 0; // смещение угла поворота датчика направления ветра.

volatile boolean blStatusChange = false; // указывает что было прерывание

long oldtimes = millis(); // время последнего изменения статуса
volatile long times = millis(); // время изменения статуса


char CSSentence[85] = "" ; //string for sentence assembly
char MWVSentence[85] = "" ; //string for MWV for sentence assembly (Wind Speed and Angle) относительная скорость ветра
int cs ; //checksum

unsigned int inZeroWindAngle = 0; // Нулевой угол паправления вертра
byte bySpeedCorrection = 100; // поправка скорости в процентах

void EEPROMWriteInt(int p_address, int p_value)
{
  byte lowByte = ((p_value >> 0) & 0xFF);
  byte highByte = ((p_value >> 8) & 0xFF);

  EEPROM.write(p_address, lowByte);
  EEPROM.write(p_address + 1, highByte);
}

//This function will read a 2 byte integer from the eeprom at the specified address and address + 1
unsigned int EEPROMReadInt(int p_address)
{
  byte lowByte = EEPROM.read(p_address);
  byte highByte = EEPROM.read(p_address + 1);

  return ((lowByte << 0) & 0xFF) + ((highByte << 8) & 0xFF00);
}



void setup()
{
 

  Serial.begin(9600,SERIAL_8N1);

  //delay(5000); //ждем 5 секунт чтоб загрузился ESP8266

  oldtimes = millis(); // обнуляем счетчики времени
  times = millis();
  pinMode(2, INPUT);


  // читаем значение смещения их EEPROM

  byte value = 0; // для значения считанного их ЕЕПРОМ
  value = EEPROM.read(0);
    if (value == 255) // если значение не установленно
    {
      value = 100;
      EEPROM.write(0, 100); // сохраняем в нем 100%
    }
  // else
  //  {
  bySpeedCorrection = value;
  //  }; // а если установленно по читаем коррекцию.

  if (EEPROMReadInt(1) < 360) { // КЕсли значение было заданно
  inZeroWindAngle = EEPROMReadInt(1); // читаем значение нулевого градуса ветра.
  }
}

void loop()
{

  /////////////////////////////////Create MWV Sentence - Wind Speed and Angle
  char b[50] = "0";
  char d[5] = "0";
  char c[50] = "0";
  int Z = 0;
  long WS = 0;

  detachInterrupt(0); // запрещаем прерывания перед обменом

  stAsk[0] = '\0';

  int i = 0;

  //если есть данные - читаем
  if (Serial.available()) {

    stAsk[0] = '\0';
    delay(100);
    //загоняем прочитанное в буфер
    while ( Serial.available() && i < 4) {
      stAsk[i++] = Serial.read();
    }
    //закрываем массив
    stAsk[i++] = '\0';
    Serial.flush(); // обнуляем сериал
    //Serial.println("UART");

  }
  if (stAsk[0] == 'Z') // если пришел сигнал на обнуление угла
  {

   // Serial.println("Zero Point");
    inZeroWindAngle = myAS5040.encoder_degrees(); //запоминаем коррекцию
    EEPROMWriteInt(0, inZeroWindAngle); // И сохраняем в ЕЕПРОМ
  }

  if (stAsk[0] == 'K' && isDigit(stAsk[1]) && isDigit(stAsk[2]) && isDigit(stAsk[3]) ) // если пришел сигнал на коррекцию скорости
  {
    bySpeedCorrection = (stAsk[1] - '0') * 100 + ((stAsk[2] - '0') * 10) + (stAsk[3] - '0'); //запоминаем коррекцию

   EEPROM.write(0, bySpeedCorrection); // И сохраняем в ЕЕПРОМ
  }

  if (blStatusChange) { // если было прерывание по датчику
   // Serial.println(times);
    WS = ((33 * bySpeedCorrection) / (times))  ; // Частота врвщения = К*V/r. Где К - коэффициент=0,379, V - скорость набегающеего потока, r-радиусот оси до центра чашечки
    // радиус крыльчатки 40мм.  скорость вращения(в радианах) = r(40mm)*2.64 (1/0.379)/t . переводим в обороты и все увеличенно в 10раз для получения десятых долей скорости
    // по новой: w(угловая скорость)= К*V/r. Где К - коэффициент=0,379, V - скорость набегающеего потока, r-радиусот оси до центра чашечки
    // V=(w*r)/K => V=((2*3,14/T)*r)/K.  T-период. V=0.66/T . ТК прериод у нас на пол оборота, то V=0.33/T 
    //все увеличенно в 10раз для получения десятых долей скорости и работе с милисекундами
    blStatusChange = false;
   // Serial.println(WS);
  } else {
    WS = 0;
  };

  //Assemble a sentance of the various parts so that we can calculate the proper checksum || Собираем стороку NMEA
  MWVSentence[0] = '\0';

  strcpy(MWVSentence, "PIMWV,"); //The string could be kept the same as what it came in as, but I chose to use the P (prefix for all proprietary devices) and I for instrument // Заголовок

  /////////////////////////////// Тут пишем направление ветра ///////////////////////////

  Z =  (myAS5040.encoder_degrees() + 360 - inZeroWindAngle) % 360; // направление ветра 0 to 359° с учетом поправки

  sprintf(b, "%d", Z);
  strcat(MWVSentence, b);
  strcat(MWVSentence, ",R,");


  ///////////////////////////// Тут Скорость ветра в метрах
  
  sprintf(d, "%d", WS/10); //d represent the meters/seconds

  sprintf(c, ".%d", (WS%10)); //c represent the deci knots

   // sprintf(c, ".%d", (WS - ((WS / 10) * 10))); //c represent the deci knots
  strcat(MWVSentence, d);
  strcat(MWVSentence, c);
  strcat(MWVSentence, ",M,A"); //the atoi command truncated the decimals, might as well stick .0 back on in case something is expecting this field to have a decimal


  ////////////////////// NMEA to serial //////////////////////////

  {
   // EtherNMEA(MWVSentence);
     Serial.end();
  Serial.begin(9600,SERIAL_8N1);
  Serial.print("$PIMWV,"); // Assemble the final message and send it out the serial port
  Serial.print(b);
  Serial.print(",R,");
  Serial.print(d);
  Serial.print(c);
  Serial.print(",M,A");
  
  //Serial.print(MWVSentence);
  Serial.print("*");
  Serial.print(Checksum(MWVSentence)); // Call Checksum function
  Serial.print("\r");
  Serial.print("\n");
  Serial.println();

   
  }
  blStatusChange = false;
  times = 0;
  attachInterrupt(0, WinSensInterupt, FALLING);  // разрешает прерывание по преднему фронту на 2 ноге для сенсора скорости ветра после обмена
//  wdt_reset();
  delay (2000);
//  wdt_reset();


}


// ////////////////////// Functions ////////////////////////////////////


String Checksum(char* CSSentence) {
  cs = 0; //clear any old checksum
  char b[50] = "0";
  //return b;

  for (int n = 0; n < strlen(CSSentence); n++) {
    cs ^= CSSentence[n]; //calculates the checksum
  }
  sprintf(b, "%02X", cs);
  return b;
}

void WinSensInterupt() { // обработка прерывания по датчику ветра

  detachInterrupt(0); // запрещаем прерывания
//Serial.println("*");
  if (times == 0) { // если сброшено время между прерываниями (очень маленькая скорость ветра)
    oldtimes = millis(); // запоминаем новое время
    blStatusChange = false; // сбрасывам флаг прерывания
    times = 10;
  }
  else
  {
    blStatusChange = true; // поднимаем флаг изменеия статуса датчика
    times = millis() - oldtimes; // запоминаем разницу во времени
    oldtimes = millis(); // запоминаем новое время
    //Serial.println(times);
  };
  attachInterrupt(0, WinSensInterupt, FALLING);  // разрешает прерывание по преднему фронту на 2 ноге для сенсора скорости ветра
}


